@@ -20,7 +20,7 @@ easy-vault - Secure vault files that are easy to use
 from __future__ import absolute_import, print_function
 
 from ._easy_vault import *  # noqa: F403,F401
-from ._key_ring_lib import *  # noqa: F403,F401
+from ._keyring import *  # noqa: F403,F401
 from ._password import *  # noqa: F403,F401
 from . import _version
 
